Python et JSON
==============
![category](développement)
![tag](python)

JSON est un format d'échange standardisé qui permet de transmettre des objets
complexe. Contrairement à SOAP par exemple, JSON n'est pas verbeux.

```json
{
    "81": {
        "day": "21/08",
        "value": "1319.8"
    },
    "87": {
        "day": "17/08",
        "value": "806.6"
    },
    "88": {
        "day": "18/08",
        "value": "879.9"
    },
    "89": {
        "day": "19/08",
        "value": "696.5"
    }
}
```

utilisation simple
------------------

Python possède un module **json** qui permet de lire et écrire du JSON.

Exemple de lecture d'un flux json

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json

url = "http://127.0.0.1/brasil/server/ca201534.json"
try: #manage python27
    import urllib2
    raw_data = urllib2.urlopen(url).read()
except: #manage python3
    import urllib
    import urllib.request
    with urllib.request.urlopen(url) as wget:
        charset = wget.info().get_content_charset() or 'utf-8'
        raw_data = wget.read().decode(charset)
val = json.loads(raw_data)
print(val['87']['value'])
print(val)
```

Exemple de production d'un flux json

```python
#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
myobj = [1, 'simple', 'list']
s = json.dumps(myobj)
print(s)
```

```note
Flask possède lui aussi ces modules json: flask.json avec les fonctions
dumps et loads
```

utilisation avec des objets complexe
------------------------------------

Création d'un objet via des données json

```python
class User(object):

    def __init__(self, name, username):
        self.name = name
        self.username = username

    def __str__(self):
        return "%s %s" % (self.name, self.username)

import json
myobj = {"name":"aoustin", "username":"fred"}
your_json = json.dumps(myobj)
j = json.loads(your_json)
u = User(**j)
print(u)
```

Un exemple plus complexe regroupant encode et decode

```python
import json
import importlib

class User(object):

    def __init__(self, name, username):
        self.name = name
        self.username = username

    def __str__(self):
        return "%s %s" % (self.name, self.username)

class MyEncoder(json.JSONEncoder):

    def default(self, obj):
        # Convert objects to a dictionary of their representation
        d = { '__class__':obj.__class__.__name__,
              '__module__':obj.__module__,
              }
        d.update(obj.__dict__)
        return d

class MyDecoder(json.JSONDecoder):

    def __init__(self):
        json.JSONDecoder.__init__(self, object_hook=self.dict_to_object)

    def dict_to_object(self, d):
        if '__class__' in d:
            class_name = d.pop('__class__')
            module_name = d.pop('__module__')
            module = importlib.import_module(module_name)
            class_ = getattr(module, class_name)
            args = dict( (key, value) for key, value in d.items())
            inst = class_(**args)
        else:
            inst = d
        return inst



obj = User('aoustin','fred')
print("obj User initial")
print(obj)
encoded_object = MyEncoder().encode(obj)
print("obj User encoder")
print(encoded_object)
myobj_instance = MyDecoder().decode(encoded_object)
print("obj User decoder")
print(myobj_instance)```
